#ifndef EventAction_h
#define EventAction_h 1

#include "G4UserEventAction.hh"
#include "TrackerIonSD.hh"
#include "TrackerGammaSD.hh"
#include "G4Event.hh"
#include "EventInformation.hh"
#include "Outgoing_Beam.hh"
#include "G4TrajectoryContainer.hh"
#include "G4Trajectory.hh"
#include "G4ios.hh"
#include "globals.hh"
#include "G4UnitsTable.hh"
#include "G4ThreeVector.hh"

#include "GEB.hh"

#define MAXDETPOS 31   // for crmat - hole indices offset+1
#define MAXCRYSTALNO 4 // for crmat
#define MAXSEGNO 8 // for average interactions

#include <fstream>
#include <string>
#include <fcntl.h>

class EventAction : public G4UserEventAction
{
public:
  EventAction();
  ~EventAction();

  void BeginOfEventAction(const G4Event*);
  void EndOfEventAction(const G4Event*);
  void SetOutFile(G4String);
  void SetOutDetsOnly() { outDetsOnly = true; }
  G4String GetOutFileName(){return outFileName;}
  G4bool EvOut(){return evOut;}
  void openEvfile();
  void closeEvfile();
  void SetMode2File(G4String);
  G4String GetMode2FileName(){return mode2FileName;}
  G4bool Mode2Out(){return mode2Out;}
  void openMode2file();
  void closeMode2file();
  void writeGEBHeader(GEBDATA*);
  void writeZeroDeg(long long int, G4double, G4double, G4double, G4double, G4double);
  void writeMINOS(long long int, G4ThreeVector, G4double);
  void writeDecomp(long long int, G4int, G4int*, G4int*, G4int*, G4double*,
		   G4double*, G4double*, G4double*, G4double*);
  void writeSim(long long int, EventInformation*);
  void SetPackRes(G4double res) { packingRes = res; }
  const G4Event* GetEvent() { return evt; }
  void SetPrint(){ print = true; }
  void SetMINOS(){ fisMINOS = true; }

  void SetInBeam(G4bool flag){fisInBeam = flag;}
  G4bool InBeam(){return fisInBeam;}
  G4bool IsMINOS(){return fisMINOS;}
  
  void SetNTotalevents(G4int n){NTotalEvents = n;}
  G4int GetNTotalevents(){return NTotalEvents;}
  void SetEveryNEvents(G4int n){everyNevents = n;}
  G4int GetEveryNEvents(){return everyNevents;}

  void SetPosRes(G4double res){posRes = res;}
  void SetThreshE(G4double e){threshE = e;}
  void SetThreshDE(G4double de){threshDE = de;}

  G4double GetAverageBetaAfter(){return averagebeta[2]/averagectr[2];}
  G4double GetAverageBetaEmission(){return averagebeta[1]/averagectr[1];}
  G4double GetAverageBetaReaction(){return averagebeta[0]/averagectr[0];}
  G4int GetAverageBetaAfterCtr(){return averagectr[2];}
  G4int GetAverageBetaEmissionCtr(){return averagectr[1];}
  G4int GetAverageBetaReactionCtr(){return averagectr[0];}
  void SetFirstInteractionFile(G4String filename){firstintfile = filename;}
  void WriteFirstInteractions();
  
private:
  G4int ionCollectionID;
  G4int gammaCollectionID;
  G4String outFileName;
  G4bool   outDetsOnly;
  std::ofstream evfile;
  G4bool evOut;
  G4String mode2FileName;
  G4int mode2file;
  G4bool mode2Out;  
  G4float crmat[MAXDETPOS][MAXCRYSTALNO][4][4];
  G4double hitTolerance;
  G4double packingRes;
  const G4Event* evt;
  G4bool print;
  G4bool fisInBeam;
  G4bool fisMINOS;
  G4int NTotalEvents;
  G4double posRes;
  G4double threshE;
  G4double threshDE;
  
  G4int timerCount;
  G4int everyNevents;
  G4double eventsPerSecond;

  G4double averagebeta[3];
  G4int averagectr[3];
  // detNum assumes always 4 crystals
  G4ThreeVector firstposition[(MBCLUST+CLOVERS)*MAXCRYSTALNO][MAXSEGNO];
  G4int firstctr[(MBCLUST+CLOVERS)*MAXCRYSTALNO][MAXSEGNO];
  G4String firstintfile;
};

#endif //EVENTACTION_H
