#ifndef GEB_h
#define GEB_h 1

#define GEB_TYPE_DECOMP         1
#define GEB_TYPE_MINIBALL       7
#define GEB_TYPE_MINOS         17
#define GEB_TYPE_ZEROPHYSDATA  13
#define GEB_TYPE_G4SIM         11
#define ZEROPHYSDATA_TYPETAG   0x0de90de9
#define MINOSXYZDATA_TYPETAG   0xaffec0c0
#define MINIBALLDATA_TYPETAG   0xf005ba11
#define SUPACLOVDATA_TYPETAG   0xdeadbeef
#define GEDECOMPDATA_TYPETAG   0xabcd5678
#define G4SIMGAMDATA_TYPETAG   0xabcd1234

#define MAX_SEGS 8              /* max. number of segments to take in events */
#define MAX_INTPTS (2*MAX_SEGS) /* max. number of interaction points */
#ifdef BACKGROUND
#define MAX_SIM_GAMMAS 500       /* max. simulated gammas per event */
#elif HIGHMULT
#define MAX_SIM_GAMMAS 40       /* max. simulated gammas per event */
#else
#define MAX_SIM_GAMMAS 10       /* max. simulated gammas per event */
#endif

#define FIRSTMB 800
#define MBSEGS 6
#define MBCLUST 8

#define CLOVERS 4
#define CLSEGS 8

typedef struct gebData
{
  int type;                     /* type of data following */
  int length;
  long long int timestamp;
} GEBDATA;

typedef union {
  GEBDATA gd;
  unsigned int header[4];
} GEBHEADER;

//interaction points
typedef struct ip{
  float x, y, z;
  float e;
  int seg;
  float seg_ener;
} IP;

typedef struct crys_ips_abcd5678 {
  int type;          /* defined as abcd5678 */
  int crystal_id;
  int num;           /* # of int pts from decomp, or # of nets on decomp error */
  float tot_e;       /* dnl corrected */
  int core_e[4];     /* 4 raw core energies from FPGA filter (no shift) */
  long long int timestamp;
  long long trig_time;    /* not yet impl */
  float t0;
  float cfd;
  float chisq;
  float norm_chisq;
  float baseline;
  float prestep;    /* avg trace value before step */
  float poststep;   /* avg trace value following step */
  int pad;          /* non-0 on decomp error, value gives error type */
  IP ips[MAX_INTPTS];
} CRYS_IPS;

typedef struct g4sim_emitted_gamma{
  float e;
  float x, y, z;
  float phi, theta;
  float beta;
} EG;

typedef struct g4sim_abcd1234 {
  int type;          /* defined as abcd1234 */
  int num;           /* # of emitted gammas */
  int full;          /* is full energy */
  EG gammas[MAX_SIM_GAMMAS];
} G4SIM_EGS;

typedef struct seg{
  int seg_id;
  float e;
} SEG;

typedef struct MB_clust{
  int type;
  int crystal_id;
  int num;
  float tot_e;
  SEG seg[MBSEGS];
} MB_DATA;

typedef struct CL_clust{
  int type;
  int crystal_id;
  int num;
  float tot_e;
  SEG seg[CLSEGS];
} CL_DATA;

typedef struct ZD_physicsdata {
  int32_t type;    /* defined 0de90de9 for indicating this version */
  float ata;    /* dispersive angle mrad       */
  float bta;    /* non-dispersive angle  mrad  */
  float xta;    /* dispersive position mm      */
  float yta;    /* non-dispersive position mm  */
  float betata; /* beta velocity after target  */
} ZD_PHYSICSDATA;

typedef struct MINOS_data {
  int32_t type;    /* defined affec0c0 for indicating this version */
  float betare;    /* beta velocity at reaction  */
  float x, y, z;   /* reaction vertex  */
} MINOS_DATA;

#endif
