#ifndef Target_H
#define Target_H 1

#include "G4Material.hh"
#include "Materials.hh"
#include "G4Box.hh"
#include "G4Tubs.hh"
#include "G4ExtrudedSolid.hh"
#include "G4IntersectionSolid.hh"
#include "G4AssemblyVolume.hh"
#include "G4LogicalVolume.hh"
#include "G4VPhysicalVolume.hh"
#include "G4ThreeVector.hh"
#include "G4PVPlacement.hh"
#include "G4VisAttributes.hh"
#include "G4Colour.hh"
#include "G4RotationMatrix.hh"
#include "G4Transform3D.hh"
#include "G4UnitsTable.hh"
#include "G4UserLimits.hh"

class Target 
{
  public:

  G4LogicalVolume *expHall_log;

  Target(G4LogicalVolume*,Materials*);
  ~Target();
  
  G4VPhysicalVolume *Construct();
  void setR(G4double);
  void setX(G4double);
  void setY(G4double);
  void setZ(G4double);
  void setMaterial(G4String);
  void setNStep(G4int);
  void Report();
  G4LogicalVolume* GetTargetLog(){return Target_log;}
  G4VSolid* GetTarget(){return aTarget;}
  G4VPhysicalVolume* GetTargetPlacement(){return Target_phys;}
  void setTargetReactionDepth(G4double);
  void ScaleDensity(G4double);
  //  void SetPositionZ(G4double);
  void SetPosition(G4double, G4double, G4double);
  G4double GetTargetThickness(){return Target_thickness;}
  G4double GetTargetArealDensity(){return Target_log->GetMaterial()->GetDensity()/(g/cm3)*Target_thickness/10;} // in g/cm^2
  G4double GetTargetNumberDensity(){return Target_log->GetMaterial()->GetDensity()/(g/cm3)/(Target_log->GetMaterial()->GetA()/(g/mole))*Target_thickness/10;} // in mole/cm^2
  G4ThreeVector* GetPos(){return Pos;}

private:
  // dimensions
  G4double Target_side_x;
  G4double Target_side_y;
  G4double Target_radius;
  G4double Target_thickness;

  Materials* materials;
  G4Material* TargetMaterial;

  //default position
  G4ThreeVector *Pos;

  bool isbox;
  G4VSolid* aTarget;

  G4LogicalVolume* Target_log;
  G4VPhysicalVolume* Target_phys;

  G4UserLimits *target_limits;

  //Number of simulation steps
  G4int NStep;

  //G4String sourceFrame;

};

#endif

