#ifndef LHTARGET
#include "Beam_Tube.hh"

Beam_Tube::Beam_Tube(G4LogicalVolume* experimentalHall_log,Materials* mat)
{
  materials=mat;
  expHall_log=experimentalHall_log;
  BTrmin = 7*cm;
  BTrmax = 7.5*cm;
  BTDz   = 100*cm; 
  BTSPhi = 0.*deg;
  BTDPhi = 360*deg;
  BTOffset = 0*cm;

  //  Pos0 = new G4ThreeVector(0.,0.,0.);
  BTPos       = new G4ThreeVector(0., 0., BTOffset);
  BeamTubeMaterial = materials->FindMaterial("Al");
  ShieldThick[0] = 1.0;
  ShieldThick[1] = 1.0;
  ShieldMaterial[0] = materials->FindMaterial("G4_Galactic");
  ShieldMaterial[1] = materials->FindMaterial("G4_Galactic");

  BeamTube = new G4Tubs("BeamTube",BTrmin,BTrmax,BTDz,BTSPhi,BTDPhi);
  BeamTube_log = new G4LogicalVolume(BeamTube,
				     BeamTubeMaterial,
				     "BeamTube_log",
				     0,0,0);

  ShieldTube[0] = new G4Tubs("Shield0Tube",BTrmax,BTrmax+ShieldThick[0],BTDz,BTSPhi,BTDPhi);
  ShieldTube[1] = new G4Tubs("Shield0Tube",BTrmax,BTrmax+ShieldThick[1],BTDz,BTSPhi,BTDPhi);
  ShieldTube_log[0] = new G4LogicalVolume(ShieldTube[0],
					    ShieldMaterial[0],
					    "Shield0Tube_log",
					    0,0,0);
  ShieldTube_log[1] = new G4LogicalVolume(ShieldTube[1],
					    ShieldMaterial[1],
					    "Shield1Tube_log",
					    0,0,0);
}

Beam_Tube::~Beam_Tube()
{;}
//-----------------------------------------------------------------------------
G4VPhysicalVolume* Beam_Tube::Construct()
{
  BeamTube = new G4Tubs("BeamTube",BTrmin,BTrmax,BTDz,BTSPhi,BTDPhi);
  
  BeamTube_log = new G4LogicalVolume(BeamTube,
				     BeamTubeMaterial,
				     "BeamTube_log",
				     0,0,0);

  BeamTube_phys = new G4PVPlacement(G4Transform3D(NoRot, *BTPos),
				    BeamTube_log,
				    "BeamTube",
				    expHall_log, false, 0);
  G4cout << "setting beam pipe -------------------------------------------------------" << G4endl;
  // Visualization Attributes

  G4Colour lblue (0.0, 1.0, 1.0, 0.3); 
  G4VisAttributes* Vis = new G4VisAttributes(lblue);
  Vis->SetVisibility(true);
  Vis->SetForceSolid(false);

  BeamTube_log->SetVisAttributes(Vis);


  return BeamTube_phys;
}
G4VPhysicalVolume* Beam_Tube::ConstructShield(G4int num)
{
  ShieldTube[num] = new G4Tubs("Shield0Tube",BTrmax,BTrmax+ShieldThick[num],BTDz,BTSPhi,BTDPhi);
  char* nameshield;
  if(num == 0)
    nameshield = (char*)"Shield0Tube_log";
  else
    nameshield = (char*)"Shield1Tube_log";
  
  ShieldTube_log[num] = new G4LogicalVolume(ShieldTube[num],
					    ShieldMaterial[num],
					    nameshield,
					    0,0,0);
  if(num == 0)
    nameshield = (char*)"Shield0Tube";
  else
    nameshield = (char*)"Shield1Tube";

  ShieldTube_phys[num] = new G4PVPlacement(G4Transform3D(NoRot, *BTPos),
					   ShieldTube_log[num],
					   nameshield,
					   expHall_log, false, 0);
  G4Colour lco (1.0, 0.5, 0.5, 0.3); 
  G4VisAttributes* VisST = new G4VisAttributes(lco);
  if(ShieldThick[num]>0)
    VisST->SetVisibility(true);
  //VisST->SetForceSolid(false);
  G4cout << num << "---------------------------->"<< ShieldThick[num] << G4endl;
  ShieldTube_log[num]->SetVisAttributes(VisST);

  return ShieldTube_phys[num];
}
//-----------------------------------------------------------------------------
void Beam_Tube::setRmin(G4double Rmin)
{
  if(Rmin<BTrmax)
    {
      BTrmin=Rmin;
      G4cout<<"----> Beam Tube inner radius set to "<<G4BestUnit(BTrmin,"Length")<< G4endl;;
      BeamTube->SetInnerRadius(BTrmin);
    }
  else
    G4cout<<"----> inner radius "<<G4BestUnit(Rmin,"Length")<<" has to be smaller than the outer radius of "<<G4BestUnit(BTrmax,"Length")<<G4endl;
}
//-----------------------------------------------------------------------------
void Beam_Tube::setRmax(G4double Rmax)
{
  if(Rmax>BTrmin)
    {
      BTrmax=Rmax;
      G4cout<<"----> Beam Tube outer radius set to "<<G4BestUnit(BTrmax,"Length")<< G4endl;;
      BeamTube->SetOuterRadius(BTrmax);
    }
  else
    G4cout<<"----> outer radius "<<G4BestUnit(Rmax,"Length")<<" has to be larger than the inner radius of "<<G4BestUnit(BTrmin,"Length")<<G4endl;
}
//-----------------------------------------------------------------------------
void Beam_Tube::setLength(G4double length)
{
      BTDz=length/2.;
      G4cout<<"----> Beam Tube length set to "<<G4BestUnit(2.*BTDz,"Length")<< G4endl;;
      BeamTube->SetZHalfLength(BTDz);
      ShieldTube[0]->SetZHalfLength(BTDz);
      ShieldTube[1]->SetZHalfLength(BTDz);

}
void Beam_Tube::setShift(G4double shift)
{
  BTPos       = new G4ThreeVector(0., 0., shift);
  BTOffset = shift;
  G4cout<<"----> Beam Tube shift set to "<<G4BestUnit(shift,"Length")<< G4endl;;
  BeamTube_phys->SetTranslation(*BTPos);
  
  ShieldTube_phys[0]->SetTranslation(*BTPos);
  ShieldTube_phys[1]->SetTranslation(*BTPos);
  // BeamTube_phys = new G4PVPlacement(G4Transform3D(NoRot, *BTPos),
  // 				    BeamTube_log,
  // 				    "BeamTube",
  // 				    expHall_log, false, 0);
  // char* nameshield;
  // for(int num=0;num<2;num++){
  //   if(num == 0)
  //     nameshield = "Shield0Tube";
  //   else
  //     nameshield = "Shield1Tube";
    
  //   ShieldTube_phys[num] = new G4PVPlacement(G4Transform3D(NoRot, *BTPos),
  // 					     ShieldTube_log[num],
  // 					     nameshield,
  // 					     expHall_log, false, 0);
    
  // }
}

void Beam_Tube::setShieldThickness(G4int num, G4double thick){
  G4cout<<"----> setting Thickness of shield" << G4endl;
  if(num>-1 && num <2){
    ShieldThick[num] = thick;
    G4cout<<"----> Thickness of shield" << num << " set to "<<G4BestUnit(ShieldThick[num],"Length")<< G4endl;;
  }
  if(num==0 && ShieldThick[0]>0){
    ShieldTube[0]->SetInnerRadius(BTrmax);
    ShieldTube[0]->SetOuterRadius(BTrmax+ShieldThick[0]);
  }
  if(num==1 && ShieldThick[1]>0){
    ShieldTube[1]->SetInnerRadius(BTrmax+ShieldThick[0]);
    ShieldTube[1]->SetOuterRadius(BTrmax+ShieldThick[0]+ShieldThick[1]);
  }
  G4Colour lco (1.0, 0.5, 0.5, 0.3); 
  G4VisAttributes* VisST = new G4VisAttributes(lco);
  if(ShieldThick[num]<=0)
    VisST->SetVisibility(false);
  if(ShieldThick[num]>0)
    VisST->SetVisibility(true);
  ShieldTube_log[num]->SetVisAttributes(VisST);

}
//-----------------------------------------------------------------------------
void Beam_Tube::Report()
{
     G4cout<<"----> Beam Tube material set to     "<<BeamTubeMaterial->GetName()<< G4endl;     
     G4cout<<"----> Beam Tube inner radius set to "<<G4BestUnit(BTrmin,"Length")<< G4endl;
     G4cout<<"----> Beam Tube outer radius set to "<<G4BestUnit(BTrmax,"Length")<< G4endl;
     G4cout<<"----> Beam Tube length set to       "<<G4BestUnit(2.*BTDz,"Length")<< G4endl;
     G4cout<<"----> Beam Tube shift set to "<<G4BestUnit(BTOffset,"Length")<< G4endl;;
     for(int i=0;i<2;i++){
       G4cout<<"----> Shield Material " << i << " set to "<<ShieldMaterial[i]->GetName()<< G4endl;;
       G4cout<<"----> Thickness of shield" << i << " set to "<<G4BestUnit(ShieldThick[i],"Length")<< G4endl;;
     }
}
//---------------------------------------------------------------------
void Beam_Tube::setMaterial(G4String materialName)
{
  // search the material by its name 
  BeamTubeMaterial = materials->FindMaterial(materialName);  
  BeamTube_log->SetMaterial(BeamTubeMaterial);
  G4cout<<" ----> Beam Tube material set to     "<<BeamTube_log->GetMaterial()->GetName()<< G4endl;                 
}
void Beam_Tube::setShieldMaterial(G4int num, G4String materialName)
{
  // search the material by its name 
  if(num>-1 && num <2 && ShieldThick[num]>0){
    ShieldMaterial[num] = materials->FindMaterial(materialName);  
    ShieldTube_log[num]->SetMaterial(ShieldMaterial[num]);
    G4cout<<" ----> Shield "<<num<<" material set to     "<<ShieldTube_log[num]->GetMaterial()->GetName()<< G4endl;                 
  }
}
#endif
