#ifndef LHTARGET
#include "Beam_Tube_Messenger.hh"

Beam_Tube_Messenger::Beam_Tube_Messenger(Beam_Tube* BT)
:BeamTube(BT)
{ 
 
  BeamTubeDir = new G4UIdirectory("/BeamTube/");
  BeamTubeDir->SetGuidance("Beam tube control.");
  
  MatCmd = new G4UIcmdWithAString("/BeamTube/Material",this);
  MatCmd->SetGuidance("Select material for the beam tube.");
  MatCmd->SetParameterName("choice",false);
  MatCmd->AvailableForStates(G4State_PreInit,G4State_Idle);
  
  RminCmd = new G4UIcmdWithADoubleAndUnit("/BeamTube/R_min",this);
  RminCmd->SetGuidance("Select the inner radius of the beam tube");
  RminCmd->SetParameterName("choice",false);
  RminCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  RmaxCmd = new G4UIcmdWithADoubleAndUnit("/BeamTube/R_max",this);
  RmaxCmd->SetGuidance("Select the outer radius of the beam tube");
  RmaxCmd->SetParameterName("choice",false);
  RmaxCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  ShiftCmd = new G4UIcmdWithADoubleAndUnit("/BeamTube/Shift",this);
  ShiftCmd->SetGuidance("Select the Z shift of the beam tube");
  ShiftCmd->SetParameterName("choice",false);
  ShiftCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  LengthCmd = new G4UIcmdWithADoubleAndUnit("/BeamTube/Length",this);
  LengthCmd->SetGuidance("Select the length of the beam tube");
  LengthCmd->SetParameterName("choice",false);
  LengthCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  Shield1MatCmd = new G4UIcmdWithAString("/BeamTube/Shield1Material",this);
  Shield1MatCmd->SetGuidance("Select material for the first shield.");
  Shield1MatCmd->SetParameterName("choice",false);
  Shield1MatCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  Shield1ThickCmd = new G4UIcmdWithADoubleAndUnit("/BeamTube/Shield1Thick",this);
  Shield1ThickCmd->SetGuidance("Select the thickness of the first shield");
  Shield1ThickCmd->SetParameterName("choice",false);
  Shield1ThickCmd->AvailableForStates(G4State_PreInit,G4State_Idle);
  
  Shield2MatCmd = new G4UIcmdWithAString("/BeamTube/Shield2Material",this);
  Shield2MatCmd->SetGuidance("Select material for the second shield.");
  Shield2MatCmd->SetParameterName("choice",false);
  Shield2MatCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  Shield2ThickCmd = new G4UIcmdWithADoubleAndUnit("/BeamTube/Shield2Thick",this);
  Shield2ThickCmd->SetGuidance("Select the thickness of the second shield");
  Shield2ThickCmd->SetParameterName("choice",false);
  Shield2ThickCmd->AvailableForStates(G4State_PreInit,G4State_Idle);
  
  RepCmd = new G4UIcmdWithoutParameter("/BeamTube/Report",this);
  RepCmd->SetGuidance("Report beam tube parameters");  

}



Beam_Tube_Messenger::~Beam_Tube_Messenger()
{
  delete LengthCmd;
  delete RminCmd;
  delete RmaxCmd;
  delete ShiftCmd;
  delete MatCmd;
  delete BeamTubeDir;
  delete Shield1MatCmd;  
  delete Shield2MatCmd;  
  delete Shield1ThickCmd;
  delete Shield2ThickCmd;
  delete RepCmd;
}


void Beam_Tube_Messenger::SetNewValue(G4UIcommand* command,G4String newValue)
{ 
  if( command == MatCmd )
    { BeamTube->setMaterial(newValue);} 
  if( command == RminCmd )
    { BeamTube->setRmin(RminCmd->GetNewDoubleValue(newValue));}
  if( command == RmaxCmd )
    { BeamTube->setRmax(RmaxCmd->GetNewDoubleValue(newValue));}
  if( command == ShiftCmd )
    { BeamTube->setShift(RmaxCmd->GetNewDoubleValue(newValue));}
  if( command == LengthCmd )
    { BeamTube->setLength(LengthCmd->GetNewDoubleValue(newValue));}
  if( command == Shield1MatCmd )
    { BeamTube->setShieldMaterial(0,newValue);} 
  if( command == Shield2MatCmd )
    { BeamTube->setShieldMaterial(1,newValue);} 
  if( command == Shield1ThickCmd )
    { BeamTube->setShieldThickness(0,Shield1ThickCmd->GetNewDoubleValue(newValue));} 
  if( command == Shield2ThickCmd )
    { BeamTube->setShieldThickness(1,Shield2ThickCmd->GetNewDoubleValue(newValue));} 
  if( command == RepCmd )
    { BeamTube->Report();}

}
#endif
