#include "EventAction.hh"
#include "RunAction.hh"

#include "G4Timer.hh"
extern G4Timer Timerintern;

EventAction::EventAction()
{ 
  ionCollectionID=-1;
  gammaCollectionID=-1;
  hitTolerance = 0.00001*mm;
  packingRes = 0.*mm;
  outFileName = "";
  outDetsOnly = false;
  evOut = false;
  mode2FileName = "";
  mode2Out = false;
  print = false;
  evt = NULL;
  fisInBeam = false;
  fisMINOS = false;
  Timerintern.Start();
  timerCount = 0;
  eventsPerSecond = 0;
  everyNevents = 1000;
  posRes = 0.;
  threshE = 0.;
  threshDE = 0.001*keV;
  for(int i=0;i<3;i++){
    averagebeta[i] = 0;
    averagectr[i] = 0;
  }
  for(int cl=0; cl<(MBCLUST+CLOVERS); cl++){
    for(int cr=0; cr<MAXCRYSTALNO; cr++){
      for(int se=0; se<MAXSEGNO; se++){
	firstposition[cl*MAXCRYSTALNO+cr][se] = G4ThreeVector(0,0,0);
	firstctr[cl*MAXCRYSTALNO+cr][se] = 0;
      }
    }
  }
  firstintfile = "";
}


EventAction::~EventAction()
{
  ;
}

void EventAction::BeginOfEventAction(const G4Event* ev)
{
  evt = ev;

  // Add a G4UserEventInformation object to store event info
  G4EventManager::
    GetEventManager()->SetUserInformation(new EventInformation);

  G4SDManager * SDman = G4SDManager::GetSDMpointer();

  if(gammaCollectionID<0||ionCollectionID<0)
    {
      gammaCollectionID=SDman->GetCollectionID("gammaCollection");
      ionCollectionID=SDman->GetCollectionID("ionCollection");
    }

  // G4cout<<"+++++ Begin of event "<<evt->GetEventID()<<G4endl;

}


void EventAction::EndOfEventAction(const G4Event* ev)
{
  evt = ev;

  ios::fmtflags f( G4cout.flags() );

  G4int event_id=evt->GetEventID();

  EventInformation* eventInfo = (EventInformation*)evt->GetUserInformation();

  // All Mode 2 output from this event gets this timestamp.
  long long int timestamp = (long long int)10000*event_id;

  if(print){
    G4cout << "-------->Mode2 data, event " << event_id << G4endl;
    if(fisInBeam)
      G4cout << std::fixed << std::setprecision(4) 
	     << std::setw(12) << std::right
	     << " ata = " << eventInfo->GetATA()
	     << " bta = " << eventInfo->GetBTA()
	     << " xta = " << eventInfo->GetXTA()
	     << " yta = " << eventInfo->GetYTA()
	     << " betata = " << eventInfo->GetBetaTA()
	     << G4endl;
    G4cout << eventInfo->GetNEmittedGammas() << " emitted gamma(s)" << G4endl;
    for(G4int i = 0; i< eventInfo->GetNEmittedGammas(); i++)
      G4cout << "energy = " << eventInfo->GetEmittedGammaEnergy(i)
	     << " pos = " << eventInfo->GetEmittedGammaPosX(i)
	     << ", " << eventInfo->GetEmittedGammaPosY(i)
	     << ", " << eventInfo->GetEmittedGammaPosZ(i)
	     << " direction = " << eventInfo->GetEmittedGammaPhi(i)
	     << ", " << eventInfo->GetEmittedGammaTheta(i)
	     << " beta = " << eventInfo->GetBeta(i)
	     << G4endl;
  }
  if(fisInBeam){
    //cout << eventInfo->GetATA() << " "<< eventInfo->GetBetaTA() << " ctr = " << averagectr[0] << " sum = " << averagebeta[0] << endl;
    if(eventInfo->GetBetaTA()>0){
      averagebeta[2]+=eventInfo->GetBetaTA();
      averagectr[2]++;
    }
    if(eventInfo->GetBetaRE()>0){
      averagebeta[0]+=eventInfo->GetBetaRE();
      averagectr[0]++;
    }
  }
  for(G4int i = 0; i< eventInfo->GetNEmittedGammas(); i++){
    averagebeta[1]+=eventInfo->GetBeta(i);
    averagectr[1]++;    
  }
  
  // Analyze hits and write event information to the output file.
  G4HCofThisEvent * HCE = evt->GetHCofThisEvent();
  if(HCE) {

    TrackerGammaHitsCollection* gammaCollection 
      = (TrackerGammaHitsCollection*)(HCE->GetHC(gammaCollectionID));

    G4int Nhits = gammaCollection->entries();

    if(Nhits>0) {

      // Packing: consolidate interaction points within segments 
      // based on proximity. 
      G4int trackID[100*MAX_INTPTS];
      G4int detNum[100*MAX_INTPTS];
      G4int segNum[100*MAX_INTPTS];
      G4double measuredEdep[100*MAX_INTPTS];
      G4double segmentEdep[100*MAX_INTPTS];
      G4double measuredX[100*MAX_INTPTS];
      G4double measuredY[100*MAX_INTPTS];
      G4double measuredZ[100*MAX_INTPTS];
      G4double X0[100*MAX_INTPTS];
      G4double Y0[100*MAX_INTPTS];
      G4double Z0[100*MAX_INTPTS];
      G4int NCons[100*MAX_INTPTS];
      G4double packingRes2 = packingRes*packingRes;

      G4int NMeasured = 0;
      G4double totalEdep = 0;

      G4bool foundfirst[(MBCLUST+CLOVERS)*MAXCRYSTALNO][MAXSEGNO];
      for(int cl=0; cl<MBCLUST+CLOVERS; cl++){
	for(int cr=0; cr<MAXCRYSTALNO; cr++){
	  for(int se=0; se<MAXSEGNO; se++){
	    foundfirst[cl*MAXCRYSTALNO+cr][se] = false;
	  }
	}
      }
      
      for(G4int i = 0; i < Nhits; i++){

	G4double x, y, z;
	x = (*gammaCollection)[i]->GetPos().getX()/mm;
	y = (*gammaCollection)[i]->GetPos().getY()/mm;
	z = (*gammaCollection)[i]->GetPos().getZ()/mm;

	G4double en  = (*gammaCollection)[i]->GetEdep()/keV;
	totalEdep += en;

	NCons[i] = -1;
	G4bool processed = false;	

	// Initialize a new interaction point for each gamma-ray hit.
	if((*gammaCollection)[i]->GetParticleID() == "gamma"){

	  // Combine multiple gamma hits at the same position.
	  // (This is rare, but it happens.)
	  if(i > 0 
	     && (x - measuredX[i-1])*(x - measuredX[i-1]) < hitTolerance*hitTolerance
	     && (y - measuredY[i-1])*(y - measuredY[i-1]) < hitTolerance*hitTolerance
	     && (z - measuredZ[i-1])*(z - measuredZ[i-1]) < hitTolerance*hitTolerance){

	    measuredEdep[NMeasured-1] += en; 
	    processed = true;

	  } else {


	    
	    

	    trackID[NMeasured]      = (*gammaCollection)[i]->GetTrackID();
	    detNum[NMeasured]       = (*gammaCollection)[i]->GetDetNumb();
	    segNum[NMeasured]       = (*gammaCollection)[i]->GetSegNumb();
	    

	    // This becomes the total energy deposit associated with this 
	    // interaction.
	    measuredEdep[NMeasured] = en; 

	    // This becomes the barycenter of all energy depositions associated
	    // with this interaction.
	    measuredX[NMeasured]    = x;
	    measuredY[NMeasured]    = y;
	    measuredZ[NMeasured]    = z;
	    trackID[NMeasured]      = (*gammaCollection)[i]->GetTrackID();

	    // Position of the initial interaction. We use position to identify
	    // the tracks produced by this interaction.
	    X0[NMeasured] = (*gammaCollection)[i]->GetPos().getX()/mm;
	    Y0[NMeasured] = (*gammaCollection)[i]->GetPos().getY()/mm;
	    Z0[NMeasured] = (*gammaCollection)[i]->GetPos().getZ()/mm;

	    //find first interaction for miniball
	    //cout << "interaction in det = " << detNum[NMeasured] << ", seg = "<< segNum[NMeasured] << endl;
	    if(detNum[NMeasured]>=FIRSTMB){
	      if(!foundfirst[detNum[NMeasured]-FIRSTMB][segNum[NMeasured]]){
//		  cout << "first interaction in det = " << detNum[NMeasured] << " - FIRSTMB " <<detNum[NMeasured]-FIRSTMB << ", seg = "<< segNum[NMeasured] << " at (x,y,z) = (" << x <<" ," << y <<" ," << z <<")" << endl;
		foundfirst[detNum[NMeasured]-FIRSTMB][segNum[NMeasured]] = true;
		G4ThreeVector gamma(x,y,z);
		firstposition[detNum[NMeasured]-FIRSTMB][segNum[NMeasured]]+=gamma;
		firstctr[detNum[NMeasured]-FIRSTMB][segNum[NMeasured]]++;
	      }
	    }

	    NCons[NMeasured] = 1;
	    NMeasured++;
	    processed = true;
	  }

	// Combine secondary-particle hits with their parent interaction points.
	} else {

	  // Compare hit i with existing "measured" interaction points.
	  for(G4int j = 0; j < NMeasured; j++){

	    G4double x0 = (*gammaCollection)[i]->GetTrackOrigin().getX()/mm;
	    G4double y0 = (*gammaCollection)[i]->GetTrackOrigin().getY()/mm;
	    G4double z0 = (*gammaCollection)[i]->GetTrackOrigin().getZ()/mm;

	    // G4cout << "(*gammaCollection)["<< i <<"]->GetParentTrackID() = "
	    // 	   << (*gammaCollection)[i]->GetParentTrackID()
	    // 	   << "   trackID[j] = " << trackID[j]
	    // 	   << "   (x0 - X0[" << j << "]) = " << (x0 - X0[j])
	    // 	   << "   (y0 - Y0[" << j << "]) = " << (y0 - Y0[j])
	    // 	   << "   (z0 - Z0[" << j << "]) = " << (z0 - Z0[j])
	    // 	   << "   (*gammaCollection)[" << i << "]->GetDetNumb()" 
	    // 	   << (*gammaCollection)[i]->GetDetNumb()
	    // 	   << "   detNum[" << j << "] = " << detNum[j] << G4endl;

	    if( (*gammaCollection)[i]->GetParentTrackID() == trackID[j]  // correct parent
		&& (x0 - X0[j])*(x0 - X0[j]) < hitTolerance*hitTolerance
		&& (y0 - Y0[j])*(y0 - Y0[j]) < hitTolerance*hitTolerance
		&& (z0 - Z0[j])*(z0 - Z0[j]) < hitTolerance*hitTolerance // correct interaction point
		&& (*gammaCollection)[i]->GetDetNumb() == detNum[j]){        // same crystal

	      // Energy-weighted average position (barycenter)
	      measuredX[j] = (measuredEdep[j]*measuredX[j] + en*x)/(measuredEdep[j] + en);
	      measuredY[j] = (measuredEdep[j]*measuredY[j] + en*y)/(measuredEdep[j] + en);
	      measuredZ[j] = (measuredEdep[j]*measuredZ[j] + en*z)/(measuredEdep[j] + en);
	      measuredEdep[j] += en;

	      NCons[j]++;
	      processed = true;
	      
	    }
	  }
	}

	// If hit i is not a gamma-ray hit and cannot be consolidated with 
	// an existing gamma-ray interaction point, it's a positron or an
	// electron multiple-scattering event that isn't associated with 
	// energy deposition in the sensitive volume of GRETINA by a gamma 
	// ray. (The gamma-ray interaction happened in dead material.)
	// We'll initialize a new interaction point and treat it as a 
	// gamma-ray interaction.
	if(!processed){

	  trackID[NMeasured]      = (*gammaCollection)[i]->GetTrackID();
	  detNum[NMeasured]       = (*gammaCollection)[i]->GetDetNumb();
	  segNum[NMeasured]       = (*gammaCollection)[i]->GetSegNumb();
	  measuredEdep[NMeasured] = en;
	  measuredX[NMeasured]    = x;
	  measuredY[NMeasured]    = y;
	  measuredZ[NMeasured]    = z;

	  // This is not a gamma ray. We need to trick its siblings into 
	  // treating it as the parent gamma.
	  trackID[NMeasured]      = (*gammaCollection)[i]->GetParentTrackID();
	  X0[NMeasured] = (*gammaCollection)[i]->GetTrackOrigin().getX()/mm;
	  Y0[NMeasured] = (*gammaCollection)[i]->GetTrackOrigin().getY()/mm;
	  Z0[NMeasured] = (*gammaCollection)[i]->GetTrackOrigin().getZ()/mm;

	  NCons[NMeasured] = 1;
	  NMeasured++;
	  processed = true;

	}

	if(!processed)
	  G4cout << "Warning: Could not find a home for hit " << i
		 << " of event " << event_id 
		 << G4endl;

	if(NMeasured >= 100*MAX_INTPTS){
	  G4cout << "Error: too many decomposed hits. Increase hit processing array dimension."
		 << G4endl;
	  exit(EXIT_FAILURE);
	}
	
      }

      // Packing: Consolidate the "measured" gamma-ray interaction points
      // within a single segment that are closer than the PackingRes 
      // parameter.
      G4int NGammaHits = NMeasured;
      for(G4int i = 0; i < NMeasured; i++){
	for(G4int j = i+1; j < NMeasured; j++){

	  if( ( (measuredX[i] - measuredX[j])*(measuredX[i] - measuredX[j])
		   + (measuredY[i] - measuredY[j])*(measuredY[i] - measuredY[j])
		   + (measuredZ[i] - measuredZ[j])*(measuredZ[i] - measuredZ[j])
		   < packingRes2 )                                  // proximal
	      && detNum[i] == detNum[j]                          // same crystal
	      && segNum[i] == segNum[j]                          // same segment
	      &&  (NCons[i] > 0 && NCons[j] > 0) ){  // not already consolidated

	    // Energy-weighted average
	    measuredX[i] = (measuredEdep[i]*measuredX[i] + measuredEdep[j]*measuredX[j])/(measuredEdep[i]+measuredEdep[j]);
	    measuredY[i] = (measuredEdep[i]*measuredY[i] + measuredEdep[j]*measuredY[j])/(measuredEdep[i]+measuredEdep[j]);
	    measuredZ[i] = (measuredEdep[i]*measuredZ[i] + measuredEdep[j]*measuredZ[j])/(measuredEdep[i]+measuredEdep[j]);
	    measuredEdep[i] += measuredEdep[j];

	    NCons[j] = -1;
	    NGammaHits--;

	  }

	}

      }

      // Calculate the total energy deposited in each segment.
      for(G4int i = 0; i < NMeasured; i++) // initialize
	if(NCons[i] > 0)
	  segmentEdep[i] = measuredEdep[i];

      G4bool singleDetector = true;
      for(G4int i = 0; i < NMeasured; i++){
	for(G4int j = i+1; j < NMeasured; j++){
	  if(NCons[i] > 0 && NCons[j] > 0
	     && detNum[i] == detNum[j]    // same crystal
	     && segNum[i] == segNum[j]){ // same segment
	    segmentEdep[i] += measuredEdep[j];
	    segmentEdep[j] += measuredEdep[i];
	  }
	  if( detNum[i] != detNum[j] )
	    singleDetector = false;
	}
      }

      // Identify events in which the full emitted gamma-ray energy
      // is deposited in a single crystal
      // (only evaluated for emitted multiplicity = 1 events).
      if( eventInfo->GetNEmittedGammas() == 1 ){
	if( singleDetector &&
	    (totalEdep - eventInfo->GetEmittedGammaEnergy(0))
	    *(totalEdep - eventInfo->GetEmittedGammaEnergy(0)) 
	    < 0.001*keV*0.001*keV )
	  eventInfo->SetFullEnergy(1);
	else
	  eventInfo->SetFullEnergy(0);
      }


      // Write event to the output file
      if(fisInBeam){
	//write out zero degree "data"
	writeZeroDeg(timestamp,eventInfo->GetATA(), eventInfo->GetBTA(), eventInfo->GetXTA(), eventInfo->GetYTA(), eventInfo->GetBetaTA());
	if(fisMINOS){
	  writeMINOS(timestamp,eventInfo->GetVertex(), eventInfo->GetBetaRE());
	}
      }

      // Fold position resolution measuredX, measuredY, measuredZ
      // WARNING: this "smears" positions across the boundaries of
      //          the active volume, which doesn't correspond to
      //          the behavior of signal decomposition.
      if(posRes > 0){
	for(int i=0; i<NMeasured; i++) {
	  measuredX[i] += CLHEP::RandGauss::shoot(0, posRes);
	  measuredY[i] += CLHEP::RandGauss::shoot(0, posRes);
	  measuredZ[i] += CLHEP::RandGauss::shoot(0, posRes);
	}
      }
      //NMeasured, detNum, segNum

      if(print){
	// Write decomposed gamma event(s) to the output file
	cout << "writing NMeasured "<<NMeasured << endl;
	for(int nn =0;nn<NMeasured;nn++){
	  cout << "detNum: " << detNum[nn] << ", segNum: " << segNum[nn] << ", Ncons " << NCons[nn];
	  //for(int jj=0;jj>NCons[nn];jj++)
	  cout << ", edep " << measuredEdep[nn] <<", segment "<<  segmentEdep[nn] << endl;
	}
      }
      writeDecomp(timestamp, 
		  NMeasured, 
		  detNum, segNum, NCons, 
		  measuredX, measuredY, measuredZ, 
		  measuredEdep, segmentEdep);
      
    }//loop hits
    
  }//hit collection

  // Write emitted gamma information from this event to 
  // the output file.

  writeSim(timestamp, eventInfo);

  if(event_id%everyNevents == 0 && event_id > 0) {
    Timerintern.Stop();
    timerCount++;
    eventsPerSecond += 
      ((double)everyNevents/Timerintern.GetRealElapsed() 
       - eventsPerSecond)/timerCount;
    G4cout << std::fixed << std::setprecision(0) << std::setw(3) 
	   << std::setfill(' ')
	   << (float)event_id/NTotalEvents*100 << " %   "
	   << eventsPerSecond << " events/s ";

    G4double hours, minutes, seconds;
    G4double time = (float)(NTotalEvents - event_id)/eventsPerSecond;
    hours = floor(time/3600.0);
    if(hours>0){
      G4cout << std::setprecision(0) << std::setw(2) 
	     << hours << ":";
      G4cout << std::setfill('0');
    } else {
      G4cout << std::setfill(' ');
    }
    minutes = floor(fmod(time,3600.0)/60.0);
    if(minutes>0){
      G4cout << std::setprecision(0) << std::setw(2) << minutes << ":";
      G4cout << std::setfill('0');
    } else {
      G4cout << std::setfill(' ');
    }
    seconds = fmod(time,60.0);
    if(seconds>0)
      G4cout << std::setprecision(0) << std::setw(2) << seconds;
    G4cout << std::setfill(' ');
    G4cout << " remaining       "
	   << "\r"<<std::flush;
    Timerintern.Start();
  }

  G4cout.flags( f );

}
// --------------------------------------------------
void EventAction::writeGEBHeader(GEBDATA* gd){

  G4int siz;

  if(print){
    GEBHEADER gh;
    gh.gd.type = gd->type;
    gh.gd.length = gd->length;
    gh.gd.timestamp = gd->timestamp;
    G4cout << "GEB header unsigned ints: ";
    for(int i=0;i<4;i++)
      G4cout << (hex) << gh.header[i] << "\t" 
	     << (dec) << gh.header[i] << G4endl;
  }

  siz = write(mode2file, (char *) gd, sizeof(GEBDATA));
  if(siz != sizeof(GEBDATA)){
    G4cout << "EventAction: error writing GEB header." << G4endl;
    G4cout << "             siz = " << siz << " (should be "
	   << sizeof(GEBDATA) << ")" << G4endl;
  }
}

// --------------------------------------------------
void EventAction::writeZeroDeg(long long int ts, G4double a, G4double b, G4double x, G4double y, G4double beta)
{
  if(mode2Out){
    //Construct GEB header for ZeroDeg event
    G4int siz;
    GEBDATA gd;
    gd.type = GEB_TYPE_ZEROPHYSDATA;
    gd.timestamp = ts;
    gd.length = sizeof(ZD_PHYSICSDATA);

    //Construct payload for ZeroDeg event
    ZD_PHYSICSDATA ZDdata;
    ZDdata.type =  ZEROPHYSDATA_TYPETAG;
    ZDdata.ata = (float)a;
    ZDdata.bta = (float)b;
    ZDdata.xta = (float)x;
    ZDdata.yta = (float)y;
    ZDdata.betata = (float)beta;

    //Write GEB header for ZeroDeg event
    writeGEBHeader(&gd);

    //Write payload for ZeroDeg event
    siz = write(mode2file, (char *) &ZDdata, gd.length);
    if(siz != gd.length){
      G4cout << "EventAction: error writing ZeroDeg GEB payload." << G4endl;
      G4cout << "             siz = " << siz << " (should be "
	     << gd.length << ")" << G4endl;
    }

  }

  if(evOut && !outDetsOnly)
    evfile << "S    " 
	   << std::fixed << std::setprecision(4) 
	   << std::right << std::setw(12) 
	   << a << std::setw(12) << b << std::setw(12) 
	   << x << std::setw(12) << y << std::setw(12) 
	   << beta << std::setw(12) 
	   << ts/10000
	   << G4endl;
}
// --------------------------------------------------
void EventAction::writeMINOS(long long int ts, G4ThreeVector pos, G4double beta)
{
  if(mode2Out){
    //Construct GEB header for ZeroDeg event
    G4int siz;
    GEBDATA gd;
    gd.type = GEB_TYPE_MINOS;
    gd.timestamp = ts;
    gd.length = sizeof(MINOS_DATA);

    //Construct payload for ZeroDeg event
    MINOS_DATA minos;
    minos.type =  MINOSXYZDATA_TYPETAG;
    minos.x = (float)pos.x();
    minos.y = (float)pos.y();
    minos.z = (float)pos.z();
    minos.betare = (float)beta;

    //Write GEB header for ZeroDeg event
    writeGEBHeader(&gd);

    //Write payload for ZeroDeg event
    siz = write(mode2file, (char *) &minos, gd.length);
    if(siz != gd.length){
      G4cout << "EventAction: error writing MINOS GEB payload." << G4endl;
      G4cout << "             siz = " << siz << " (should be "
	     << gd.length << ")" << G4endl;
    }

  }

  if(evOut && !outDetsOnly)
    evfile << "S    " 
	   << std::fixed << std::setprecision(4) 
	   << std::right << std::setw(12) 
	   << pos.x() << std::setw(12) << pos.y() << std::setw(12) 
	   << pos.z() << std::setw(12) << beta << std::setw(12) 
	   << ts/10000
	   << G4endl;
}
// --------------------------------------------------
void EventAction::writeDecomp(long long int ts, 
			      G4int NMeasured, 
			      G4int detNum[], G4int segNum[], G4int NCons[],
			      G4double x[], G4double y[], G4double z[], 
			      G4double e[], G4double se[])
{
  G4int siz;
  GEBDATA gd;
  CRYS_IPS crys_ips[100*MAX_INTPTS];
  MB_DATA mb_data[100];
  
  G4int NGEtrack = 0;
  G4int NMBclust = 0;
  G4bool Processed[100*MAX_INTPTS];
  for(G4int i = 0; i < NMeasured; i++)
    Processed[i] = false;

  for(G4int i = 0; i < NMeasured; i++){
    if( NCons[i] > 0 && Processed[i] == false ){
      //cout << " in writing detNum["<<i<<"] " << detNum[i] << endl;
      if(detNum[i]<FIRSTMB){//Getrack
	crys_ips[NGEtrack].type = GEDECOMPDATA_TYPETAG;
	crys_ips[NGEtrack].crystal_id = detNum[i]+4; // +4 to match measured data
	crys_ips[NGEtrack].num = 1;
	crys_ips[NGEtrack].tot_e = e[i]; //NOT += for the first one!
	
	crys_ips[NGEtrack].core_e[0] = 0;
	crys_ips[NGEtrack].core_e[1] = 0;
	crys_ips[NGEtrack].core_e[2] = 0;
	crys_ips[NGEtrack].core_e[3] = 0;
	crys_ips[NGEtrack].timestamp = ts;
	crys_ips[NGEtrack].t0 = 0;
	crys_ips[NGEtrack].cfd = 0;
	crys_ips[NGEtrack].chisq = 0;
	crys_ips[NGEtrack].norm_chisq = 0;
	crys_ips[NGEtrack].baseline = 0;
	crys_ips[NGEtrack].prestep = 0;
	crys_ips[NGEtrack].poststep = 0;
	crys_ips[NGEtrack].pad = 0;
	crys_ips[NGEtrack].ips[ crys_ips[NGEtrack].num-1 ].x = x[i];
	crys_ips[NGEtrack].ips[ crys_ips[NGEtrack].num-1 ].y = y[i];
	crys_ips[NGEtrack].ips[ crys_ips[NGEtrack].num-1 ].z = z[i];
	crys_ips[NGEtrack].ips[ crys_ips[NGEtrack].num-1 ].e = e[i];
	crys_ips[NGEtrack].ips[ crys_ips[NGEtrack].num-1 ].seg = segNum[i];
	crys_ips[NGEtrack].ips[ crys_ips[NGEtrack].num-1 ].seg_ener = se[i];
	Processed[i] = true;
	
	for(G4int j = i+1; j < MAX_INTPTS; j++){ // Clear the interaction points
	  crys_ips[NGEtrack].ips[j].x        = 0.;
	  crys_ips[NGEtrack].ips[j].y        = 0.;
	  crys_ips[NGEtrack].ips[j].z        = 0.;
	  crys_ips[NGEtrack].ips[j].e        = 0.;
	  crys_ips[NGEtrack].ips[j].seg      = 0;
	  crys_ips[NGEtrack].ips[j].seg_ener = 0.;
	}
	
	// Get other interactions with this crystal
	for(G4int j = i+1; j < NMeasured; j++){
	  if(detNum[j]<FIRSTMB){
	    if(NCons[j] > 0 && Processed[j] == false && detNum[j]+4 == crys_ips[NGEtrack].crystal_id){
	      crys_ips[NGEtrack].tot_e += e[j];
	      // Only MAX_INTPTS interaction points can be stored in a crys_ips.
	      if( crys_ips[NGEtrack].num < MAX_INTPTS ){
		crys_ips[NGEtrack].ips[ crys_ips[NGEtrack].num ].x = x[j];
		crys_ips[NGEtrack].ips[ crys_ips[NGEtrack].num ].y = y[j];
		crys_ips[NGEtrack].ips[ crys_ips[NGEtrack].num ].z = z[j];
		crys_ips[NGEtrack].ips[ crys_ips[NGEtrack].num ].e = e[j];
		crys_ips[NGEtrack].ips[ crys_ips[NGEtrack].num ].seg = segNum[j];
		crys_ips[NGEtrack].ips[ crys_ips[NGEtrack].num ].seg_ener = se[j];
	      }
	      crys_ips[NGEtrack].num++; // Check for overflow below, and warn.
	      Processed[j] = true;
	    }
	  }
	}
	NGEtrack++;
      }//Ge tracking det data
      else{// miniball and glovers
	mb_data[NMBclust].type = MINIBALLDATA_TYPETAG;
	mb_data[NMBclust].crystal_id = detNum[i]+4; // +4 to match measured data
	mb_data[NMBclust].num = 1;
	mb_data[NMBclust].tot_e = e[i]; //NOT += for the first one!
	mb_data[NMBclust].seg[mb_data[NMBclust].num-1].seg_id = segNum[i];
	mb_data[NMBclust].seg[mb_data[NMBclust].num-1].e = se[i];
	for(G4int j = i+1; j < MBSEGS; j++){ // Clear the segment data
	  mb_data[NMBclust].seg[j].seg_id = 0;
	  mb_data[NMBclust].seg[j].e = 0;	  
	}
	// Get other interactions with this crystal
	for(G4int j = i+1; j < NMeasured; j++){
	  if(detNum[j]>=FIRSTMB){
	    if(NCons[j] > 0 && Processed[j] == false && detNum[j]+4 == mb_data[NMBclust].crystal_id){
	      mb_data[NMBclust].tot_e += e[j];
	      //check if the segment was already set
	      bool alreadyset = false;
	      for(int k=0; k<mb_data[NMBclust].num; k++){
		if(mb_data[NMBclust].seg[k].seg_id == segNum[j])
		  alreadyset = true;
	      }
	      if(!alreadyset){
		mb_data[NMBclust].seg[mb_data[NMBclust].num].seg_id = segNum[j];
		mb_data[NMBclust].seg[mb_data[NMBclust].num].e = se[j];
		mb_data[NMBclust].num++; // Check for overflow below, and warn.
	      }
	      Processed[j] = true;
	    }
	  }
	}
	NMBclust++;
      }//miniball and clovers
    }//not done yet
  }//measured

  if(mode2Out){
    //Construct GEB header for decomp event(s)
    gd.type = GEB_TYPE_DECOMP;
    gd.timestamp = ts;
    gd.length = sizeof(CRYS_IPS);

    for(G4int i = 0; i < NGEtrack; i++){

      if(crys_ips[i].num > MAX_INTPTS){
	G4cout << "Warning: " << crys_ips[i].num << " interaction points."
	       << "         only " << MAX_INTPTS << " can be written."
	       << G4endl;
	crys_ips[i].num = MAX_INTPTS;

      }

      //Write GEB header for decomp event
      writeGEBHeader(&gd);

      //Write GEB payload for decomp event
      siz = write(mode2file, (char *) &crys_ips[i], sizeof(CRYS_IPS));
      if(siz != sizeof(CRYS_IPS)){
	G4cout << "EventAction: error writing decomp GEB payload." << G4endl;
	G4cout << "             siz = " << siz << " (should be "
	       << sizeof(CRYS_IPS) << ")" << G4endl;
      }

    }//NGEtrack
    gd = GEBDATA(); // reset
    gd.type = GEB_TYPE_MINIBALL;
    gd.timestamp = ts;
    gd.length = sizeof(MB_DATA);
    for(G4int i = 0; i < NMBclust; i++){
      if(mb_data[i].num > MBSEGS){
	G4cout << "Warning: " << mb_data[i].num << " segments points."
	       << "         only " << MBSEGS << " can be written."
	       << G4endl;
	mb_data[i].num = MBSEGS;	
      }

      //Write GEB header for miniball event
      writeGEBHeader(&gd);

      //Write GEB payload for miniball event
      siz = write(mode2file, (char *) &mb_data[i], sizeof(MB_DATA));
      if(siz != sizeof(MB_DATA)){
	G4cout << "EventAction: error writing miniball GEB payload." << G4endl;
	G4cout << "             siz = " << siz << " (should be "
	       << sizeof(MB_DATA) << ")" << G4endl;
      }
    }//NMBclust
  }//mode2

  if(evOut){
    evfile << "D" << std::setw(4) << NGEtrack 
    	   << std::setw(12) << ts/10000 << G4endl;
    for(G4int i = 0; i < NGEtrack; i++){
      evfile << "C" << std::setw(4) << crys_ips[i].crystal_id
	     << std::setw(4) << crys_ips[i].num << G4endl;
      for(G4int j = 0; j < crys_ips[i].num; j++){
	evfile << std::setw(5) 
	       << crys_ips[i].ips[j].seg 
	       << std::fixed << std::setprecision(4) 
	       << std::right << std::setw(12) 
	       << crys_ips[i].ips[j].e << std::setw(12) 
	       << crys_ips[i].ips[j].x << std::setw(12) 
	       << crys_ips[i].ips[j].y << std::setw(12) 
	       << crys_ips[i].ips[j].z
	       << G4endl;
      }
    }
  }

}
// --------------------------------------------------
void EventAction::writeSim(long long int ts, EventInformation* eventInfo)
{
  if(mode2Out){
    G4int siz;
    GEBDATA gd;
    G4SIM_EGS g4sim_egs;

    //Construct GEB header for G4SIM event
    gd.type = GEB_TYPE_G4SIM;
    gd.timestamp = ts;
    gd.length = sizeof(G4SIM_EGS);

    //Construct GEB payload for G4SIM event
    g4sim_egs.type = G4SIMGAMDATA_TYPETAG;
    g4sim_egs.num = eventInfo->GetNEmittedGammas();
    g4sim_egs.full = eventInfo->GetFullEnergy();

    for(G4int i = 0; i < g4sim_egs.num; i++){
      g4sim_egs.gammas[i].e     = eventInfo->GetEmittedGammaEnergy(i);
      g4sim_egs.gammas[i].x     = eventInfo->GetEmittedGammaPosX(i);
      g4sim_egs.gammas[i].y     = eventInfo->GetEmittedGammaPosY(i);
      g4sim_egs.gammas[i].z     = eventInfo->GetEmittedGammaPosZ(i);
      g4sim_egs.gammas[i].phi   = eventInfo->GetEmittedGammaPhi(i);
      g4sim_egs.gammas[i].theta = eventInfo->GetEmittedGammaTheta(i);
      g4sim_egs.gammas[i].beta  = eventInfo->GetBeta(i);
    }

    //Write GEB header for G4SIM event
    writeGEBHeader(&gd);

    //Write GEB payload for G4SIM event
    siz = write(mode2file, (char *) &g4sim_egs, sizeof(G4SIM_EGS));
    if(siz != sizeof(G4SIM_EGS)){
      G4cout << "EventAction: error writing G4SIM GEB payload." << G4endl;
      G4cout << "             siz = " << siz << " (should be " 
	     << sizeof(G4SIM_EGS) << ")" << G4endl;
    }

  }

  if(evOut && !outDetsOnly){
    evfile << "E" << std::setw(4) << eventInfo->GetNEmittedGammas()  
	   << std::setw(4) << eventInfo->GetFullEnergy()  
	   << std::setw(12) << ts/10000 << G4endl;
    for(G4int i = 0; i < eventInfo->GetNEmittedGammas(); i++){
      evfile << "     "
	     << std::fixed << std::setprecision(4) 
	     << std::right << std::setw(12) 
	     << eventInfo->GetEmittedGammaEnergy(i)
	     << std::setw(12) 
	     << eventInfo->GetEmittedGammaPosX(i)
	     << std::setw(12) 
	     << eventInfo->GetEmittedGammaPosY(i)
	     << std::setw(12) 
	     << eventInfo->GetEmittedGammaPosZ(i)
	     << std::setw(12) 
	     << eventInfo->GetEmittedGammaPhi(i)
	     << std::setw(12) 
	     << eventInfo->GetEmittedGammaTheta(i)
	     << std::setw(12)
	     << eventInfo->GetBeta(i) << G4endl;
    }
  }
 
}
// --------------------------------------------------TB
void EventAction::openEvfile()
{
  if (!evfile.is_open()) evfile.open(outFileName.c_str());
  if (!evfile.is_open()){
    G4cout<< "ERROR opening evfile." << G4endl;
    evOut = false;
  } else {
    G4cout << "\nOpened output file: " << outFileName << G4endl;
    evOut = true;
  }
  return;
}
//--------------------------------------------------- TB
void EventAction::closeEvfile()
{
  evfile.close();
  return;
}
//----------------------------------------------------TB
void EventAction::SetOutFile(G4String name)
{
  outFileName = name;
  closeEvfile();
  openEvfile();
  return;
}
// --------------------------------------------------
void EventAction::openMode2file()
{
  mode2file = open(mode2FileName.c_str(), O_WRONLY | O_CREAT | O_TRUNC, 
		   S_IRUSR | S_IWUSR | S_IRGRP |  S_IWGRP | S_IROTH);
  if(mode2file < 0){
    G4cout << "ERROR opening mode2file" << G4endl;
    mode2Out = false;
  } else {
    G4cout << "\nOpened mode 2 Output file: " << mode2FileName << G4endl;
    mode2Out = true;
  }
  return;
}
//---------------------------------------------------
void EventAction::closeMode2file()
{
  close(mode2file);
  return;
}
//----------------------------------------------------
void EventAction::SetMode2File(G4String name)
{
  mode2FileName = name;
  openMode2file();
  return;
}
//---------------------------------------------------
void EventAction::WriteFirstInteractions(){

 ofstream fout;
  if(firstintfile=="")
    return;
  fout.open(firstintfile);
  for(int cl=0; cl<MBCLUST+CLOVERS; cl++){
    for(int cr=0; cr<MAXCRYSTALNO; cr++){
      for(int se=0; se<MAXSEGNO; se++){
	if(firstctr[cl*MAXCRYSTALNO+cr][se]<1)
	  continue;
	//cout << "cluster " << cl <<", crystal " << cr << ", segment " << se << ", index " << cl*MAXCRYSTALNO+cr;
	firstposition[cl*MAXCRYSTALNO+cr][se] /= firstctr[cl*MAXCRYSTALNO+cr][se];
	//cout << "\ttheta " <<firstposition[cl*MAXCRYSTALNO+cr][se].theta()<< ",\tphi " <<firstposition[cl*MAXCRYSTALNO+cr][se].phi() << "\tctr " << firstctr[cl*MAXCRYSTALNO+cr][se] << endl;
	fout << "Miniball.Clu"<<cl<<".Cry"<<cr<<".Seg"<<se<<".X:\t"<<firstposition[cl*MAXCRYSTALNO+cr][se].x() << endl;
	fout << "Miniball.Clu"<<cl<<".Cry"<<cr<<".Seg"<<se<<".Y:\t"<<firstposition[cl*MAXCRYSTALNO+cr][se].y() << endl;
	fout << "Miniball.Clu"<<cl<<".Cry"<<cr<<".Seg"<<se<<".Z:\t"<<firstposition[cl*MAXCRYSTALNO+cr][se].z() << endl;
	fout << "Miniball.Clu"<<cl<<".Cry"<<cr<<".Seg"<<se<<".Theta:\t"<<firstposition[cl*MAXCRYSTALNO+cr][se].theta() << endl;
	fout << "Miniball.Clu"<<cl<<".Cry"<<cr<<".Seg"<<se<<".Phi:\t"<<firstposition[cl*MAXCRYSTALNO+cr][se].phi() << endl;
	fout << "Miniball.Clu"<<cl<<".Cry"<<cr<<".Seg"<<se<<".Ctr:\t"<<firstctr[cl*MAXCRYSTALNO+cr][se] << endl;
      }
    }
  }
  fout.close();
  
}
