#ifndef LHTARGET
#include "Target.hh"

Target::Target(G4LogicalVolume* experimentalHall_log,Materials* mat)
{
  materials=mat;
  expHall_log=experimentalHall_log;
  isbox = true;
  Target_radius=25*mm;
  Target_side_x=50*mm;
  Target_side_y=50*mm;
  Target_thickness=0.1*mm;
  TargetMaterial = materials->FindMaterial("G4_Galactic");

  Pos = new G4ThreeVector(0.,0.,0.);
  NStep=20;

}

Target::~Target()
{ delete target_limits;}

//-----------------------------------------------------------------------------
G4VPhysicalVolume* Target::Construct()
{
  G4cout << "construct target" << G4endl;
  aTarget = new G4Box("target",Target_side_x/2.,Target_side_y/2.,Target_thickness/2.);
  isbox = true;

  Target_log = new G4LogicalVolume(aTarget,TargetMaterial,"Target_log",0,0,0);
  target_limits= new G4UserLimits();
  target_limits->SetMaxAllowedStep(Target_thickness/NStep);
  Target_log->SetUserLimits(target_limits);

  Target_phys = new G4PVPlacement(G4Transform3D(G4RotationMatrix::IDENTITY,*Pos),Target_log,"Target",expHall_log,false,0);

  //G4Colour blue (0.0, 0.0, 1.0); 
  G4Colour grey (0.5, 0.5, 0.5); 
  G4VisAttributes* Vis_6 = new G4VisAttributes(grey);
  if(TargetMaterial->GetName() == "G4_Galactic"){
    Vis_6->SetVisibility(false);
  } else {
    Vis_6->SetVisibility(true);
    Vis_6->SetForceSolid(true);
  }

  Target_log->SetVisAttributes(Vis_6);

  return Target_phys;
}

//-----------------------------------------------------------------------------
void Target::setR(G4double R)
{
   Target_radius=R;
   isbox = false;
   aTarget = new G4Tubs("target",0,Target_radius,Target_thickness/2.,0*deg,360*deg);
   Target_log->SetSolid(aTarget);
   Target_phys->SetLogicalVolume(Target_log);
   G4cout<<"----> Target radius is set to "<<G4BestUnit(((G4Tubs*)aTarget)->GetOuterRadius(),"Length")<<G4endl;
}
//-----------------------------------------------------------------------------
void Target::setX(G4double X)
{
   Target_side_x=X;
   isbox = true;
   ((G4Box*)aTarget)->SetXHalfLength(Target_side_x/2.);
   G4cout<<"----> Target side X is set to "<<G4BestUnit(2.*((G4Box*)aTarget)->GetXHalfLength(),"Length")<<G4endl;
}
//-----------------------------------------------------------------------------
void Target::setY(G4double Y)
{
   Target_side_y=Y;
   isbox = true;
   ((G4Box*)aTarget)->SetYHalfLength(Target_side_y/2.);
   G4cout<<"----> Target side Y is set to "<<G4BestUnit(2.*((G4Box*)aTarget)->GetYHalfLength(),"Length")<<G4endl;
}
//-----------------------------------------------------------------------------
void Target::setZ(G4double Z)
{
  Target_thickness=Z;
  if(isbox)
    ((G4Box*)aTarget)->SetZHalfLength(Target_thickness/2.);
  else
    ((G4Tubs*)aTarget)->SetZHalfLength(Target_thickness/2.);
  target_limits->SetMaxAllowedStep(Target_thickness/NStep);
  Target_log->SetUserLimits(target_limits);
  G4cout<<"----> Target thickness is set to ";
  if(isbox)
    G4cout <<G4BestUnit(2.*((G4Box*)aTarget)->GetZHalfLength(),"Length")<<G4endl;
  else
    G4cout <<G4BestUnit(2.*((G4Tubs*)aTarget)->GetZHalfLength(),"Length")<<G4endl;
}
//-----------------------------------------------------------------------------
void Target::setNStep(G4int n)
{
   NStep=n;
   target_limits->SetMaxAllowedStep(Target_thickness/NStep);
   Target_log->SetUserLimits(target_limits);
   G4cout<<"----> Number of simulation steps in the target is set to "<<NStep<<G4endl;
}

//-----------------------------------------------------------------------------
void Target::Report()
{
  G4cout<<"----> Target material set to  "<<Target_log->GetMaterial()->GetName()<< G4endl;   
  G4cout<<"----> Target density:         "<<Target_log->GetMaterial()->GetDensity()/(g/cm3) << " g/cm3"<< G4endl;
  if(isbox){
    G4cout<<"----> Target side X is set to "<<G4BestUnit(2.*((G4Box*)aTarget)->GetXHalfLength(),"Length")<<G4endl;
    G4cout<<"----> Target side Y is set to "<<G4BestUnit(2.*((G4Box*)aTarget)->GetYHalfLength(),"Length")<<G4endl;  
    G4cout<<"----> Target thickness is set to "<<G4BestUnit(2.*((G4Box*)aTarget)->GetZHalfLength(),"Length")<<G4endl;
  }
  else{
    G4cout<<"----> Target radius is set to "<<G4BestUnit(((G4Tubs*)aTarget)->GetOuterRadius(),"Length")<<G4endl;
    G4cout<<"----> Target thickness is set to "<<G4BestUnit(2.*((G4Tubs*)aTarget)->GetZHalfLength(),"Length")<<G4endl;
  }
  G4cout<<"----> Number of simulation steps in the target is set to "<<NStep<<G4endl;
}


//---------------------------------------------------------------------
void Target::setMaterial(G4String materialName)
{
  if(materialName == "MINOS"){
    isbox=false;
    aTarget = new G4Tubs("target",0,Target_radius,Target_thickness/2.,0*deg,360*deg);
    Target_log->SetSolid(aTarget);
    Target_phys->SetLogicalVolume(Target_log);
  }
  // search the material by its name 
  TargetMaterial = materials->FindMaterial(materialName);  
  Target_log->SetMaterial(TargetMaterial);
  G4cout<<"----> Target material set to     "<<Target_log->GetMaterial()->GetName()<< G4endl;        

  //G4Colour blue (0.0, 0.0, 1.0); 
  G4Colour grey (0.5, 0.5, 0.5); 
  G4VisAttributes* Vis_6 = new G4VisAttributes(grey);
  if(TargetMaterial->GetName() == "G4_Galactic"){
    Vis_6->SetVisibility(false);
  } else {
    Vis_6->SetVisibility(true);
    Vis_6->SetForceSolid(true);
  }

  Target_log->SetVisAttributes(Vis_6);
         
}
//-------------------------------------------------------------------
void Target::setTargetReactionDepth(G4double depth)
{
  //  G4cout<<"\n----> The depth is "<<G4BestUnit(depth,"Length")<< G4endl;;
  target_limits->SetUserMinRange(depth);
}
//-----------------------------------------------------------------------------
void Target::SetPosition(G4double x, G4double y, G4double z)
{

  G4ThreeVector Sep;
  Sep.setX(Pos->getX() + x);
  Sep.setY(Pos->getY() + y);
  Sep.setZ(Pos->getZ() + z);
  Target_phys->SetTranslation(Sep);

  x=Target_phys->GetTranslation().getX();
  y=Target_phys->GetTranslation().getY();
  z=Target_phys->GetTranslation().getZ();

  Pos->setX(x);
  Pos->setY(y);
  Pos->setZ(z);

  G4cout <<"----> Target position is set to "
	 << G4BestUnit(x,"Length") << ", "
    	 << G4BestUnit(y,"Length") << ", "
	 << G4BestUnit(z,"Length") << ", "
	 << G4endl;
}
//---------------------------------------------------------------------
void Target::ScaleDensity(G4double scale)
{
  // search the material by its name 
  G4String name=TargetMaterial->GetName();
  G4double Z=TargetMaterial->GetZ();
  G4double A=TargetMaterial->GetA();
  G4double density=TargetMaterial->GetDensity();
  density*=scale;
  TargetMaterial=new G4Material(name, Z,A,density);
  Target_log->SetMaterial(TargetMaterial);
  G4cout<<"----> Target material set to     "<<Target_log->GetMaterial()->GetName()<< G4endl;  
  G4cout<<"----> Target Z set to            "<<Target_log->GetMaterial()->GetZ()<< G4endl;  
  G4cout<<"----> Target mole mass set to       "<<Target_log->GetMaterial()->GetA()/g*mole<<" g/mole"<< G4endl;  
  G4cout<<"----> Target density set to         "<<Target_log->GetMaterial()->GetDensity()/g*cm3<<" g/cm3"<< G4endl;    
             
}
//-------------------------------------------------------------------
#endif
